import 'dart:async';

import 'package:path/path.dart' as path;

import 'package:daily_programmer_client_dart/src/util/entity_name.dart';
import 'package:daily_programmer_client_dart/src/generator/generator.dart';

/// Generator for challenge.
class ChallengeGenerator extends Generator {
  static const _templateFolder = 'challenge';
  static final List<String> _templateFileNames = [
    path.join('lib', 'challenge.dart.mustache'),
    path.join('test', 'challenges', 'challenge_test.dart.mustache'),
  ];

  /// Class name of this challenge.
  final String className;

  /// Separator.
  final String selector;

  /// Challenge file name without extension.
  final String targetName;

  /// Dry run flag set from cli
  final bool dryRun;

  /// Return an instance of [ChallengeGenerator] with the context of parent of
  /// command.
  factory ChallengeGenerator(
          EntityName classEntityName, String destinationFolder,
          {bool dryRun: false}) =>
      new ChallengeGenerator._(
          classEntityName.camelCased,
          classEntityName.dashed,
          classEntityName.underscored,
          destinationFolder,
          dryRun);

  ChallengeGenerator._(this.className, this.selector, this.targetName,
      String destinationFolder, this.dryRun)
      : super(destinationFolder);

  // Gets a map from template file name to target file name.
  Map<String, String> _getTemplateTargetPaths() {
    final results = <String, String>{};
    for (final templateFileName in _templateFileNames) {
      results[path.join(_templateFolder, templateFileName)] = templateFileName
          .replaceAll('challenge_test.dart', '${targetName}_test.dart')
          .replaceAll('challenge.dart', '$targetName.dart')
          .replaceAll('.mustache', '');
    }
    return results;
  }

  @override
  Future generate() async {
    await renderAndWriteTemplates(_getTemplateTargetPaths(), dryRun: dryRun);
  }
}
