import 'dart:async' show Future;
import 'dart:isolate' show Isolate;

import 'package:path/path.dart' as path;

/// Fixes invalid path.
/// For example:
///   `path\to/folder` -> `path/to/folder` (posix)
String getNormalizedPath(String oldPath) {
  if (oldPath == null) throw new ArgumentError('Path is null.');

  final regExp = new RegExp(r'[\\/]');
  return oldPath.startsWith(regExp)
      ? '/${path.normalize(path.joinAll(oldPath.split(regExp)))}'.trim()
      : path.normalize(path.joinAll(oldPath.split(regExp))).trim();
}

/// Converts `\` in [uri] into `/`.
String fixUri(String uri) {
  if (uri == null) throw new ArgumentError('Uri is null.');
  return uri.replaceAll('\\', '/');
}

/// Extracts package name from package [uri].
String getPackageName(String uri) {
  if (uri == null || uri.isEmpty)
    throw new ArgumentError('Uri is null or empty.');
  if (!uri.contains(new RegExp(r'(dart|package):[A-Za-z0-9\/_-]+\.dart')))
    throw new ArgumentError("$uri isn't valid package path.");
  return uri
      .substring(0, uri.indexOf('/'))
      .replaceAll('dart:', '')
      .replaceAll('package:', '');
}

/// Helper function for resolving to a non-relative, non-package URI.
Future<Uri> resolveUri(Uri uri) =>
    Isolate.resolvePackageUri(uri ?? Uri.base).then((resolvedUri) {
      if (resolvedUri == null || uri == null) {
        throw new ArgumentError.value(uri, 'uri', 'Unknown package');
      } else if (!uri.scheme.contains(new RegExp(r'dart|package'))) {
        throw new ArgumentError.value(
            uri.scheme, 'scheme', 'Not package or dart');
      } else {
        return resolvedUri;
      }
    });
